/*  Pgn.cpp: Pgn export functions
 *
 *  Feringa is a chess game for the Palm Pilot based on OpenChess.
 *  Copyright (C) 2005, Manfred Joerg.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <PalmOS.h>
#include <StringMgr.h>
#include "gnuchess2.h"
#include "OpenChess.h"
#include "Pgn.hxx"

void WriteCoordinates(char *Buffer, const UInt8 uiValue)
{
   Buffer[0] = uiValue % 8 + 'a';
   Buffer[1] = uiValue / 8 + '1';
   Buffer[2] = ' ';
   Buffer[3] = 0;
}


void WriteMove(char *Buffer, const short i)
{
   Char sPiece[] = { 'N', 'B', 'R', 'Q', 'K' };
	UInt8 f,t;
	f = GameList[i].gmove>>8;
	t = GameList[i].gmove & 0xFF;

	if (GameList[i].fpiece == pawn)
	{
		UInt8 Index;
		if (GameList[i].piece == 0)
		{
			WriteCoordinates(Buffer, t);
			Index = 2;
		}
		else
		{
			Buffer[0] = 'a' + f % 8;
			Buffer[1] = 'x';
			WriteCoordinates(Buffer+2, t);
			Index = 4;
		}
		if (t / 8 == 0 || t / 8 == 7)
		{
			Buffer[Index] = 'D';
			Buffer[Index+1] = ' ';
			Buffer[Index+2] = 0;
		}
	}
	else if ((GameList[i].fpiece == king) && (f%8 == 4) && (t%8 == 6))
	{
		StrCopy(Buffer, "O-O ");
		Buffer[4] = 0;
	}
	else if ((GameList[i].fpiece == king) && (f%8 == 4) && (t%8 == 2))
	{
		StrCopy(Buffer, "O-O-O ");
		Buffer[6] = 0;
	}
	else
	{
		Buffer[0] = sPiece[GameList[i].fpiece - 2];
      WriteCoordinates(Buffer+1, f);
		if (GameList[i].piece == 0)
			WriteCoordinates(Buffer+3, t);
		else
		{
			Buffer[3] = 'x';
			WriteCoordinates(Buffer+4, t);
		}
	}
}


Err ExportPgn()
{
	DmOpenRef ref;
   MemHandle record;
   MemPtr RecPtr;
   Char Buffer[35];
   short i;
   UInt16 Offset = 0;
   UInt16 recordSize = 100;
   UInt16 recordIndex = dmMaxRecordIndex;

   ref = DmOpenDatabaseByTypeCreator('DATA', 'memo', dmModeReadWrite);
   if (ref == NULL)
   	return DmGetLastErr();

   record = DmNewRecord(ref, &recordIndex, recordSize);
	RecPtr = MemHandleLock(record);

   char  NameWhite[21];
   char  NameBlack[21];
   UInt8 Result;
   if (true == GetGameParameters(NameWhite, NameBlack, &Result))
   {
   	StrPrintF(Buffer, "[White \"%s\"]\n", NameWhite);
      DmStrCopy(RecPtr, Offset, Buffer);
      Offset += StrLen(Buffer);
      StrPrintF(Buffer, "[Black \"%s\"]\n", NameBlack);
      DmStrCopy(RecPtr, Offset, Buffer);
      Offset += StrLen(Buffer);

      switch (Result)
      {
      	case 0:
         	StrCopy(Buffer, "[Result \"*\"]\n");
            break;
         case 1:
         	StrCopy(Buffer, "[Result \"1-0\"]\n");
            break;
         case 2:
         	StrCopy(Buffer, "[Result \"1/2-1/2\"]\n");
            break;
         case 3:
         	StrCopy(Buffer, "[Result \"0-1\"]\n");
            break;
      }
      DmStrCopy(RecPtr, Offset, Buffer);
      Offset += StrLen(Buffer);
   }

   for (i = 0; i <= GameCnt; i++)
   {
      if (Offset + 20 > recordSize)
		{
			MemHandleUnlock(record);
			record = DmResizeRecord(ref, recordIndex, recordSize + 20);
			if (record == NULL)
				return DmGetLastErr();
			RecPtr = MemHandleLock(record);
			recordSize += 20;
		}

		if (i % 2 == 0)
		{
			StrPrintF(Buffer, "%d. ", i / 2 + 1);
			DmStrCopy(RecPtr, Offset, Buffer);
			Offset += StrLen(Buffer);
		}

		WriteMove(Buffer, i);
      DmStrCopy(RecPtr, Offset, Buffer);
      Offset += StrLen(Buffer);
   }

   MemHandleUnlock(record);
   DmReleaseRecord(ref, recordIndex, true);
   DmCloseDatabase(ref);
   return errNone;
}
